<?php

namespace Drupal\google_one_tap\Controller;

use Drupal\Component\Render\FormattableMarkup;
use Drupal\user\Controller\UserAuthenticationController as UserController;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpKernel\Exception\BadRequestHttpException;

/**
 * Returns responses for Google One Tap routes.
 */
class UserAuthenticationController extends UserController {

  /**
   * Handles sign-up/sign-in via Google One Tap.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *
   * @return \Symfony\Component\HttpFoundation\Response
   */
  public function login(Request $request) {

    // Get token.
    if (!$token = $request->request->get('idToken')) {
      throw new BadRequestHttpException('Missing idToken');
    }
    // Get Google Client.
    $client = new \Google_Client(['client_id' => $this->config('google_one_tap.configuration')->get('client_id')]);

    // Verify token.
    $payload = $client->verifyIdToken($token);
    if (!$payload) {
      throw new BadRequestHttpException('Invalid Token');
    }

    // @todo: Limit to G Suite.
    //$domain = $payload['hd'];

    if (empty($payload['email']) || !\Drupal::service('email.validator')->isValid($payload['email'])) {
      throw new BadRequestHttpException('Incorrect email address');
    }

    if (!$user = user_load_by_mail($payload['email'])) {
      $this
        ->getLogger('google_one_tap')
        ->notice('Registering a new user for %mail', [
          '%mail' => $payload['email'],
        ]);
      // Register the user.
      $edit = [
        'name' => $this->findNextAvailableAccountName($payload['name']),
        'mail' => $payload['email'],
        'status' => 1,
      ];
      /** @var \Drupal\user\UserInterface $new_user */
      $new_user = $this->userStorage->create($edit);
      $new_user->save();
      $user = $new_user;
    }

    if ($this->userIsBlocked($user->getEmail())) {
      throw new BadRequestHttpException('The user has not been activated or is blocked.');
    }

    $this->userLoginFinalize($user);

    // Compose basic response.
    // @see Drupal\user\Controller\UserAuthenticationController::login()
    $response_data = [];
    if ($user->get('uid')->access('view', $user)) {
      $response_data['current_user']['uid'] = $user->id();
    }
    if ($user->get('roles')->access('view', $user)) {
      $response_data['current_user']['roles'] = $user->getRoles();
    }
    if ($user->get('name')->access('view', $user)) {
      $response_data['current_user']['name'] = $user->getAccountName();
    }
    $response_data['csrf_token'] = $this->csrfToken->get('rest');

    $logout_route = $this->routeProvider->getRouteByName('user.logout.http');
    // Trim '/' off path to match \Drupal\Core\Access\CsrfAccessCheck.
    $logout_path = ltrim($logout_route->getPath(), '/');
    $response_data['logout_token'] = $this->csrfToken->get($logout_path);

    return new JsonResponse($response_data);
  }

  /**
   * Verifies if the user is blocked.
   *
   * @param string $email
   *
   * @return bool
   */
  protected function userIsBlocked($email) {
    return (bool) \Drupal::entityQuery('user')
      ->condition('mail', $email)
      ->condition('status', 0)
      ->execute();
  }

  /**
   * Resolves first available username.
   *
   * @param $name
   *
   * @return \Drupal\Component\Render\FormattableMarkup
   */
  protected function findNextAvailableAccountName($name) {
    $count = 0;
    $old_name = $name;
    while ($this->accountNameIsRegistered($name)) {
      $count++;
      $name = new FormattableMarkup('@name_@count', ['@name' => $old_name, '@count' => $count]);
    };
    return $name;
  }

  /**
   * Verifies if username is already registered.
   *
   * @param string $name
   *
   * @return bool
   */
  protected function accountNameIsRegistered($name) {
    return (bool) \Drupal::entityQuery('user')
      ->condition('name', $name)
      ->execute();
  }
}
